#include <avr/io.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>

#include "usbdrv.h"
#include "oddebug.h"


#define TICKS_2MS = F_CPU / 500

// http://www.avrfreaks.net/index.php?name=PNphpBB2&file=printview&t=50106
// http://awawa.hariko.com/avr_lcd_drive_en.html

void showchar(char c);
void showstring(char *str);

char phase;
char dataA[4];
char dataB[4];
char playsymbol = 0;

char calcdata;

char hackaday = 0;
char hmem1 = 0;

uchar	usbFunctionSetup(uchar data[8]) {
  static uchar replyBuf[4];
  usbMsgPtr = replyBuf;
  uchar len = (data[1] & 3)+1;       // 1 .. 4 bytes 
  uchar target = (data[1] >> 3) & 3; // target 0 .. 3
  uchar i;

  // request byte:

  // 7 6 5 4 3 2 1 0
  // C C C T T R L L

  // TT = target bit map 
  // R = reserved for future use, set to 0
  // LL = number of bytes in transfer - 1 

  switch(data[1] >> 5) {
	case 0: // echo (for transfer reliability testing)
		replyBuf[0] = data[2];
		replyBuf[1] = data[3];
		return 2;
		break;
	case 2: // data
		if(hackaday)
			return;
		if(target == 1) {
			dataA[0] = (data[2] & ~1) | (data[5] & 1);
			dataA[1] = (data[3] & ~1) | ((data[5] & 2) >> 1);
			dataA[2] = (data[4] & ~1) | ((data[5] & 4) >> 2);
			playsymbol = (data[5] & 8) >> 3;

			//dataA[3] = data[5]-1;
		}
		if(target == 2) {
			if(((data[2] & 0xFF) == 0xF7)) {
				hackaday = 1;
				return;
			}
			dataB[0] = (data[2] & ~1) | (data[5] & 1);
			dataB[1] = (data[3] & ~1) | ((data[5] & 2) >> 1);
			dataB[2] = (data[4] & ~1) | ((data[5] & 4) >> 2);
			//dataB[3] = data[5]-1;
		}
		break;
	default:
		break;
  }
  return 0;  // reply len
}

static void initForUsbConnectivity(void)
{
uchar   i = 0;

#if F_CPU == 12800000
    TCCR0 = 3;          /* 1/64 prescaler */
#endif
    usbInit();
    /* enforce USB re-enumerate: */
    usbDeviceDisconnect();  /* do this while interrupts are disabled */
    do{             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }while(--i);
    usbDeviceConnect();
    sei();
}



// PD0 = COM0 = pin 29
// PD1 = COM1 = pin 30

ISR(TIMER1_COMPA_vect) {
	// do stuff
	switch(phase) {
		case 0:
			DDRD &= ~(1 << 0); // set as input
			PORTD &= ~(1 << 0); // clear pull up
			
			DDRD |= (1 << 1); // output
			PORTD &= ~(1 << 1); // 0

			PORTD &= ~(1 << 7); // 0
			
			if(playsymbol)
				PORTD |= (1 << 6);  // 1
			else
				PORTD &= ~(1 << 6); // 0

			PORTA = dataB[0];
			PORTB = dataB[1];
			PORTC = dataB[2];
			
			phase++;
		break;
		case 1:
			DDRD &= ~(1 << 0); // set as input
			PORTD &= ~(1 << 0); // clear pull up

			DDRD |= (1 << 1); // output
			PORTD |= (1 << 1); // 1

			PORTD |= (1 << 7);  // 1 , PD7 = common "off" signal for all additional segments
			if(playsymbol) // play symbol is connected to PD6
				PORTD &= ~(1 << 6); // 0
			else
				PORTD |= (1 << 6);  // 1

			PORTA = ~(dataB[0]); // PORTA..PORTC are connected to all the number segments and some special segments (shuffle, repeat, ...)
			PORTB = ~(dataB[1]);
			PORTC = ~(dataB[2]);

			phase++;
		break;
		case 2:
			DDRD |= (1 << 0); // output
			PORTD &= ~(1 << 0); // 0

			DDRD &= ~(1 << 1); // set as input
			PORTD &= ~(1 << 1); // clear pull up
			PORTD |= (1 << 1); // set pull up

			PORTD &= ~(1 << 7); // 0
			if(playsymbol)
				PORTD |= (1 << 6);  // 1
			else
				PORTD &= ~(1 << 6); // 0

			PORTA = dataA[0];
			PORTB = dataA[1];
			PORTC = dataA[2];
			
			phase++;
		break;
		case 3:
			DDRD |= (1 << 0); // output
			PORTD |= (1 << 0); // 1

			DDRD &= ~(1 << 1); // set as input
			PORTD &= ~(1 << 1); // clear pull up

			PORTD |= (1 << 7);  // 1
			if(playsymbol)
				PORTD &= ~(1 << 6); // 0
			else
				PORTD |= (1 << 6);  // 1

			PORTA = ~(dataA[0]);
			PORTB = ~(dataA[1]);
			PORTC = ~(dataA[2]);

			phase = 0;
		break;
	}
}


int __attribute__((noreturn)) main(void)
{
	
	
	dataA[0] = 0xFF;
	dataB[0] = 0xFF;
	dataA[1] = 0xFF;
	dataB[1] = 0xFF;
	dataA[2] = 0xFF;
	dataB[2] = 0xFF;

	DDRA = 0xFF; // output
	PORTA = 0;
	DDRB = 0xFF; // output
	PORTB = 0;

	DDRC = 0xFF; // output
	PORTC = 0;
	DDRD |= (1 << 7); // output
	DDRD |= (1 << 6); // output

	
//	odDebugInit();

	phase = 0;

	TCCR1B = (1 << WGM12); // Configure timer 1 for CTC mode 

	TIMSK |= (1 << OCIE1A); // Enable CTC interrupt 

	// (enable global interrupts)
	initForUsbConnectivity();

	//OCR1A = 23999;
	OCR1A = 47999;

	TCCR1B |= (1 << CS10); // Set up timer with prescale 1

	do {
		wdt_reset();
		DDRA = 0xFF; // output
		DDRB = 0xFF; // output
		DDRC = 0xFF; // output
		if(hackaday) {
			dataA[0] = 0xb1;
			dataA[1] = 0xd0;
			dataA[2] = 0xdb;
			dataB[0] = 0x63; // mid
			dataB[1] = 0xf0;
			dataB[2] = 0xf2;
			_delay_ms(700);
			dataA[0] = 0xd9;
			dataA[1] = 0x0b-1; // right
			dataA[2] = 0x00;
			dataB[0] = 0x77; // mid
			dataB[1] = 0x87;
			dataB[2] = 0x80;
			_delay_ms(700);
			dataA[0] = 0x99;
			dataA[1] = 0x03;
			dataA[2] = 0x00;
			dataB[0] = 0x37; // mid
			dataB[1] = 0x83;
			dataB[2] = 0x80;
			_delay_ms(700);
			dataA[0] = 0xf7;
			dataA[1] = 0xe7;
			dataA[2] = 0x00;
			dataB[0] = 0x51; // mid
			dataB[1] = 0xc4;
			dataB[2] = 0x80;
			_delay_ms(700);
			hackaday = 0;
		}
		usbPoll();
	} while(1);


}

